@@ -7,12 +7,10 @@ from orangewidget.settings import Setting
 from oasys.widgets import gui as oasysgui, congruence
 from oasys.widgets.exchange import DataExchangeObject
 
-from collections import OrderedDict
-from orangecontrib.xoppy.util import srundplug
-import scipy.constants as codata
-import os
+
 
 from orangecontrib.xoppy.widgets.gui.ow_xoppy_widget import XoppyWidget
+from orangecontrib.xoppy.util.xoppy_undulators import xoppy_calc_undulator_spectrum
 
 from syned.widget.widget_decorator import WidgetDecorator
 import syned.beamline.beamline as synedb
@@ -39,16 +37,23 @@ class OWundulator_spectrum(XoppyWidget, WidgetDecorator):
     PERIODID = Setting(0.018)
     NPERIODS = Setting(222)
     KV = Setting(1.68)
+    KH = Setting(0.0)
+    KPHASE = Setting(0.0)
     DISTANCE = Setting(30.0)
     GAPH = Setting(0.001)
     GAPV = Setting(0.001)
+    GAPH_CENTER = Setting(0.0)
+    GAPV_CENTER = Setting(0.0)
     PHOTONENERGYMIN = Setting(3000.0)
     PHOTONENERGYMAX = Setting(55000.0)
     PHOTONENERGYPOINTS = Setting(500)
-    METHOD = Setting(0)
+    METHOD = Setting(2)
 
     inputs = WidgetDecorator.syned_input_data()
 
+    def __init__(self):
+        super().__init__(show_script_tab=True)
+
     def build_gui(self):
 
         box = oasysgui.widgetBox(self.controlArea, self.name + " Input Parameters", orientation="vertical", width=self.CONTROL_AREA_WIDTH-5)
@@ -153,6 +158,25 @@ class OWundulator_spectrum(XoppyWidget, WidgetDecorator):
                     valueType=float, validator=QDoubleValidator(), orientation="horizontal", labelWidth=250)
         self.show_at(self.unitFlags()[idx], box1)
 
+        #widget index 9B
+        idx += 1
+        box1 = gui.widgetBox(box)
+        self.id_KH = oasysgui.lineEdit(box1, self, "KH",
+                     label=self.unitLabels()[idx], addSpace=False,
+                    valueType=float, validator=QDoubleValidator(), orientation="horizontal", labelWidth=250)
+        self.show_at(self.unitFlags()[idx], box1)
+
+        #widget index 9C
+        idx += 1
+        box1 = gui.widgetBox(box)
+        self.id_KPHASE = oasysgui.lineEdit(box1, self, "KPHASE",
+                     label=self.unitLabels()[idx], addSpace=False,
+                    valueType=float, validator=QDoubleValidator(), orientation="horizontal", labelWidth=250)
+        self.show_at(self.unitFlags()[idx], box1)
+
+
+
+
         #widget index 10
         idx += 1
         box1 = gui.widgetBox(box)
@@ -177,6 +201,22 @@ class OWundulator_spectrum(XoppyWidget, WidgetDecorator):
                     valueType=float, validator=QDoubleValidator(), orientation="horizontal", labelWidth=250)
         self.show_at(self.unitFlags()[idx], box1)
 
+        #widget index 11B
+        idx += 1
+        box1 = gui.widgetBox(box)
+        oasysgui.lineEdit(box1, self, "GAPH_CENTER",
+                     label=self.unitLabels()[idx], addSpace=False,
+                    valueType=float, validator=QDoubleValidator(), orientation="horizontal", labelWidth=250)
+        self.show_at(self.unitFlags()[idx], box1)
+
+        #widget index 12B
+        idx += 1
+        box1 = gui.widgetBox(box)
+        oasysgui.lineEdit(box1, self, "GAPV_CENTER",
+                     label=self.unitLabels()[idx], addSpace=False,
+                    valueType=float, validator=QDoubleValidator(), orientation="horizontal", labelWidth=250)
+        self.show_at(self.unitFlags()[idx], box1)
+
         #widget index 13
         idx += 1
         box1 = gui.widgetBox(box)
@@ -210,11 +250,26 @@ class OWundulator_spectrum(XoppyWidget, WidgetDecorator):
                     valueType=int, orientation="horizontal", labelWidth=250)
         self.show_at(self.unitFlags()[idx], box1)
 
+
     def unitLabels(self):
-         return ["Use emittances","Electron Energy [GeV]", "Electron Energy Spread", "Electron Current [A]", "Electron Beam Size H [m]", "Electron Beam Size V [m]", "Electron Beam Divergence H [rad]", "Electron Beam Divergence V [rad]", "Period ID [m]", "Number of periods", "Kv [undulator K value vertical field]", "Distance to slit [m]", "Slit gap H [m]", "Slit gap V [m]", "photon Energy Min [eV]", "photon Energy Max [eV]", "photon Energy Points", "calculation code"]
+         return ["Use emittances","Electron Energy [GeV]", "Electron Energy Spread",
+                 "Electron Current [A]", "Electron Beam Size H [m]", "Electron Beam Size V [m]",
+                 "Electron Beam Divergence H [rad]", "Electron Beam Divergence V [rad]", "Period ID [m]",
+                 "Number of periods", "Kv [K value vertical field]", "Kh [K value horizontal field]",
+                 "Kphase [phase diff Kh - Kv in rad]","Distance to slit [m]","Slit gap H [m]",
+                 "Slit gap V [m]", "Slit center H [m]", "Slit center V [m]",
+                 "photon Energy Min [eV]","photon Energy Max [eV]",
+                 "photon Energy Points", "calculation code"]
 
     def unitFlags(self):
-         return ["True", "True", "self.USEEMITTANCES == 1 and self.METHOD != 1", "True", "self.USEEMITTANCES == 1", "self.USEEMITTANCES == 1", "self.USEEMITTANCES == 1", "self.USEEMITTANCES == 1", "True", "True", "True", "True", "True", "True", "True", "True", "True", "True"]
+         return ["True", "True", "self.USEEMITTANCES == 1 and self.METHOD != 1",
+                 "True", "self.USEEMITTANCES == 1", "self.USEEMITTANCES == 1",
+                 "self.USEEMITTANCES == 1", "self.USEEMITTANCES == 1", "True",
+                 "True", "True", "self.METHOD != 0",
+                 "self.METHOD != 0","True", "True",
+                 "True", "True", "True",
+                 "True", "True",
+                 "True", "True"]
 
     def get_help_name(self):
         return 'undulator_spectrum'
@@ -230,9 +285,13 @@ class OWundulator_spectrum(XoppyWidget, WidgetDecorator):
         self.PERIODID = congruence.checkStrictlyPositiveNumber(self.PERIODID, "Period ID")
         self.NPERIODS = congruence.checkStrictlyPositiveNumber(self.NPERIODS, "Number of Periods")
         self.KV = congruence.checkPositiveNumber(self.KV, "Kv")
+        self.KH = congruence.checkPositiveNumber(self.KH, "Kh")
+        self.KPHASE = congruence.checkNumber(self.KPHASE, "KPHASE")
         self.DISTANCE = congruence.checkPositiveNumber(self.DISTANCE, "Distance to slit")
         self.GAPH = congruence.checkPositiveNumber(self.GAPH, "Slit gap H")
         self.GAPV = congruence.checkPositiveNumber(self.GAPV, "Slit gap V")
+        self.GAPH_CENTER = congruence.checkPositiveNumber(self.GAPH_CENTER, "Slit center H")
+        self.GAPV_CENTER = congruence.checkPositiveNumber(self.GAPV_CENTER, "Slit center V")
         self.PHOTONENERGYMIN = congruence.checkPositiveNumber(self.PHOTONENERGYMIN, "photon Energy Min")
         self.PHOTONENERGYMAX = congruence.checkStrictlyPositiveNumber(self.PHOTONENERGYMAX, "photon Energy Max")
         congruence.checkLessThan(self.PHOTONENERGYMIN, self.PHOTONENERGYMAX, "photon Energy Min", "photon Energy Max")
@@ -245,7 +304,7 @@ class OWundulator_spectrum(XoppyWidget, WidgetDecorator):
         #     raise Exception("SRW calculation code not supported under Linux")
 
     def do_xoppy_calculation(self):
-        return xoppy_calc_undulator_spectrum(ELECTRONENERGY=self.ELECTRONENERGY,
+        energy, flux, spectral_power, cumulated_power = xoppy_calc_undulator_spectrum(ELECTRONENERGY=self.ELECTRONENERGY,
                                              ELECTRONENERGYSPREAD=self.ELECTRONENERGYSPREAD,
                                              ELECTRONCURRENT=self.ELECTRONCURRENT,
                                              ELECTRONBEAMSIZEH=self.ELECTRONBEAMSIZEH,
@@ -255,22 +314,102 @@ class OWundulator_spectrum(XoppyWidget, WidgetDecorator):
                                              PERIODID=self.PERIODID,
                                              NPERIODS=self.NPERIODS,
                                              KV=self.KV,
+                                             KH=self.KH,
+                                             KPHASE=self.KPHASE,
                                              DISTANCE=self.DISTANCE,
                                              GAPH=self.GAPH,
                                              GAPV=self.GAPV,
+                                             GAPH_CENTER=self.GAPH_CENTER,
+                                             GAPV_CENTER=self.GAPV_CENTER,
                                              PHOTONENERGYMIN=self.PHOTONENERGYMIN,
                                              PHOTONENERGYMAX=self.PHOTONENERGYMAX,
                                              PHOTONENERGYPOINTS=self.PHOTONENERGYPOINTS,
                                              METHOD=self.METHOD,
                                              USEEMITTANCES=self.USEEMITTANCES)
 
-    def extract_data_from_xoppy_output(self, calculation_output):
-        e, f, sp = calculation_output
+        # write python script in standard output
+        dict_parameters = {
+            "ELECTRONENERGY" : self.ELECTRONENERGY,
+            "ELECTRONENERGYSPREAD" : self.ELECTRONENERGYSPREAD,
+            "ELECTRONCURRENT" : self.ELECTRONCURRENT,
+            "ELECTRONBEAMSIZEH" : self.ELECTRONBEAMSIZEH,
+            "ELECTRONBEAMSIZEV" : self.ELECTRONBEAMSIZEV,
+            "ELECTRONBEAMDIVERGENCEH" : self.ELECTRONBEAMDIVERGENCEH,
+            "ELECTRONBEAMDIVERGENCEV" : self.ELECTRONBEAMDIVERGENCEV,
+            "PERIODID" : self.PERIODID,
+            "NPERIODS" : self.NPERIODS,
+            "KV" : self.KV,
+            "KH": self.KH,
+            "KPHASE": self.KPHASE,
+            "DISTANCE" : self.DISTANCE,
+            "GAPH" : self.GAPH,
+            "GAPV" : self.GAPV,
+            "GAPH_CENTER": self.GAPH_CENTER,
+            "GAPV_CENTER": self.GAPV_CENTER,
+            "PHOTONENERGYMIN" : self.PHOTONENERGYMIN,
+            "PHOTONENERGYMAX" : self.PHOTONENERGYMAX,
+            "PHOTONENERGYPOINTS" : self.PHOTONENERGYPOINTS,
+            "METHOD" : self.METHOD,
+            "USEEMITTANCES" : self.USEEMITTANCES,
+        }
 
-        data = numpy.zeros((len(e), 3))
+
+        # print(self.script_template().format_map(dict_parameters))
+
+        self.xoppy_script.set_code(self.script_template().format_map(dict_parameters))
+
+        return energy, flux, spectral_power, cumulated_power
+
+    def script_template(self):
+        return """
+#
+# script to make the calculations (created by XOPPY:undulator_spectrum)
+#
+from orangecontrib.xoppy.util.xoppy_undulators import xoppy_calc_undulator_spectrum
+energy, flux, spectral_power, cumulated_power = xoppy_calc_undulator_spectrum(
+    ELECTRONENERGY={ELECTRONENERGY},
+    ELECTRONENERGYSPREAD={ELECTRONENERGYSPREAD},
+    ELECTRONCURRENT={ELECTRONCURRENT},
+    ELECTRONBEAMSIZEH={ELECTRONBEAMSIZEH},
+    ELECTRONBEAMSIZEV={ELECTRONBEAMSIZEV},
+    ELECTRONBEAMDIVERGENCEH={ELECTRONBEAMDIVERGENCEH},
+    ELECTRONBEAMDIVERGENCEV={ELECTRONBEAMDIVERGENCEV},
+    PERIODID={PERIODID},
+    NPERIODS={NPERIODS},
+    KV={KV},
+    KH={KH},
+    KPHASE={KPHASE},
+    DISTANCE={DISTANCE},
+    GAPH={GAPH},
+    GAPV={GAPV},
+    GAPH_CENTER={GAPH_CENTER},
+    GAPV_CENTER={GAPV_CENTER},
+    PHOTONENERGYMIN={PHOTONENERGYMIN},
+    PHOTONENERGYMAX={PHOTONENERGYMAX},
+    PHOTONENERGYPOINTS={PHOTONENERGYPOINTS},
+    METHOD={METHOD},
+    USEEMITTANCES={USEEMITTANCES})
+# example plot
+from srxraylib.plot.gol import plot
+plot(energy,flux,ytitle="Flux [photons/s/o.1%bw]",xtitle="Poton energy [eV]",title="Undulator Flux",
+    xlog=False,ylog=False,show=False)
+plot(energy,spectral_power,ytitle="Power [W/eV]",xtitle="Poton energy [eV]",title="Undulator Spectral Power",
+    xlog=False,ylog=False,show=False)
+plot(energy,cumulated_power,ytitle="Cumulated Power [W]",xtitle="Poton energy [eV]",title="Undulator Cumulated Power",
+    xlog=False,ylog=False,show=True)
+#
+# end script
+#
+"""
+
+    def extract_data_from_xoppy_output(self, calculation_output):
+        e, f, sp, csp = calculation_output
+
+        data = numpy.zeros((len(e), 4))
         data[:, 0] = numpy.array(e)
         data[:, 1] = numpy.array(f)
         data[:, 2] = numpy.array(sp)
+        data[:, 3] = numpy.array(csp)
 
         calculated_data = DataExchangeObject("XOPPY", self.get_data_exchange_widget_name())
         calculated_data.add_content("xoppy_data", data)
@@ -281,19 +420,22 @@ class OWundulator_spectrum(XoppyWidget, WidgetDecorator):
         return "UNDULATOR_FLUX"
 
     def getTitles(self):
-        return ['Undulator Flux', 'Spectral Power']
+        return ['Undulator Flux', 'Spectral Power', 'Cumulated Power']
 
     def getXTitles(self):
-        return ["Energy [eV]", "Energy [eV]"]
+        return ["Energy [eV]", "Energy [eV]", "Energy [eV]"]
 
     def getYTitles(self):
-        return ["Flux [Phot/sec/0.1%bw]", "Spectral Power [W/eV]"]
+        return ["Flux [Phot/sec/0.1%bw]", "Spectral Power [W/eV]", "Cumulated Power [W]"]
 
     def getLogPlot(self):
-        return [(False, False), (False, False)]
+        return [(False, False), (False, False), (False, False)]
 
     def getVariablesToPlot(self):
-        return [(0, 1), (0, 2)]
+        return [(0, 1), (0, 2), (0, 3)]
+
+    def getDefaultPlotTabIndex(self):
+        return 0
 
     def receive_syned_data(self, data):
 
@@ -312,8 +454,10 @@ class OWundulator_spectrum(XoppyWidget, WidgetDecorator):
                 self.ELECTRONBEAMDIVERGENCEH = xp
                 self.ELECTRONBEAMDIVERGENCEV = yp
                 self.PERIODID = light_source._magnetic_structure._period_length
-                self.NPERIODS = light_source._magnetic_structure._number_of_periods
+                self.NPERIODS = int(light_source._magnetic_structure._number_of_periods)
                 self.KV = light_source._magnetic_structure._K_vertical
+                self.KH = light_source._magnetic_structure._K_horizontal
+                # TODO  self.KPHASE = light_source._magnetic_structure....
 
                 self.set_enabled(False)
 
@@ -336,6 +480,7 @@ class OWundulator_spectrum(XoppyWidget, WidgetDecorator):
                 self.id_PERIODID.setEnabled(True)
                 self.id_NPERIODS.setEnabled(True)
                 self.id_KV.setEnabled(True)
+                self.id_KH.setEnabled(True)
         else:
                 self.id_ELECTRONENERGY.setEnabled(False)
                 self.id_ELECTRONENERGYSPREAD.setEnabled(False)
@@ -347,80 +492,8 @@ class OWundulator_spectrum(XoppyWidget, WidgetDecorator):
                 self.id_PERIODID.setEnabled(False)
                 self.id_NPERIODS.setEnabled(False)
                 self.id_KV.setEnabled(False)
-# --------------------------------------------------------------------------------------------
-# --------------------------------------------------------------------------------------------
+                self.id_KH.setEnabled(False)
 
-def xoppy_calc_undulator_spectrum(ELECTRONENERGY=6.04,ELECTRONENERGYSPREAD=0.001,ELECTRONCURRENT=0.2,\
-                              ELECTRONBEAMSIZEH=0.000395,ELECTRONBEAMSIZEV=9.9e-06,\
-                              ELECTRONBEAMDIVERGENCEH=1.05e-05,ELECTRONBEAMDIVERGENCEV=3.9e-06,\
-                              PERIODID=0.018,NPERIODS=222,KV=1.68,DISTANCE=30.0,GAPH=0.001,GAPV=0.001,\
-                              PHOTONENERGYMIN=3000.0,PHOTONENERGYMAX=55000.0,PHOTONENERGYPOINTS=500,METHOD=0,
-                              USEEMITTANCES=1):
-    print("Inside xoppy_calc_undulator_spectrum. ")
-
-    bl = OrderedDict()
-    bl['ElectronBeamDivergenceH'] = ELECTRONBEAMDIVERGENCEH
-    bl['ElectronBeamDivergenceV'] = ELECTRONBEAMDIVERGENCEV
-    bl['ElectronBeamSizeH'] = ELECTRONBEAMSIZEH
-    bl['ElectronBeamSizeV'] = ELECTRONBEAMSIZEV
-    bl['ElectronCurrent'] = ELECTRONCURRENT
-    bl['ElectronEnergy'] = ELECTRONENERGY
-    bl['ElectronEnergySpread'] = ELECTRONENERGYSPREAD
-    bl['Kv'] = KV
-    bl['NPeriods'] = NPERIODS
-    bl['PeriodID'] = PERIODID
-    bl['distance'] = DISTANCE
-    bl['gapH'] = GAPH
-    bl['gapV'] = GAPV
-
-    if USEEMITTANCES:
-        zero_emittance = False
-    else:
-        zero_emittance = True
-
-    #TODO remove file and export e,f arrays
-    outFile = "undulator_spectrum.spec"
-
-    codata_mee = codata.m_e * codata.c**2 / codata.e # electron mass in eV
-    gamma = bl['ElectronEnergy'] * 1e9 / codata_mee
-
-    m2ev = codata.c * codata.h / codata.e      # lambda(m)  = m2eV / energy(eV)
-    resonance_wavelength = (1 + bl['Kv']**2 / 2.0) / 2 / gamma**2 * bl["PeriodID"]
-    resonance_energy = m2ev / resonance_wavelength
-    print ("Gamma: %f \n"%(gamma))
-    print ("Resonance wavelength [A]: %g \n"%(1e10*resonance_wavelength))
-    print ("Resonance energy [eV]: %g \n"%(resonance_energy))
-
-
-    if METHOD == 0:
-        print("Undulator flux calculation using US. Please wait...")
-        e, f = srundplug.calc1d_us(bl,photonEnergyMin=PHOTONENERGYMIN,photonEnergyMax=PHOTONENERGYMAX,
-              photonEnergyPoints=PHOTONENERGYPOINTS,fileName=outFile,fileAppend=False,zero_emittance=zero_emittance)
-        print("Done")
-        print("\nCheck calculation output at: %s"%(os.path.join(os.getcwd(),"us.out")))
-    if METHOD == 1:
-        print("Undulator flux calculation using URGENT. Please wait...")
-        e, f = srundplug.calc1d_urgent(bl,photonEnergyMin=PHOTONENERGYMIN,photonEnergyMax=PHOTONENERGYMAX,
-              photonEnergyPoints=PHOTONENERGYPOINTS,fileName=outFile,fileAppend=False,zero_emittance=zero_emittance)
-        print("Done")
-        print("\nCheck calculation output at: %s"%(os.path.join(os.getcwd(),"urgent.out")))
-    if METHOD == 2:
-        # get the maximum harmonic number
-        h_max = int(1.1*PHOTONENERGYMAX/resonance_energy)
-
-        print ("Number of harmonics considered: %d \n"%(h_max))
-        print("Undulator flux calculation using SRW. Please wait...")
-        e, f = srundplug.calc1d_srw(bl,photonEnergyMin=PHOTONENERGYMIN,photonEnergyMax=PHOTONENERGYMAX,
-              photonEnergyPoints=PHOTONENERGYPOINTS,fileName=outFile,fileAppend=False,zero_emittance=zero_emittance,
-              srw_max_harmonic_number=h_max)
-        print("Done")
-
-    if zero_emittance:
-        print("\nNo emittance calculation")
-
-    print("\nPower from integral of spectrum: %8.3f W"%(f.sum()*1e3*codata.e*(e[1]-e[0])))
-
-    return e, f, f*codata.e * 1e3
 
 
 
